/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdbool.h>
#include <Elementary.h>
#include <app_debug.h>
#include <inputmgr.h>

#include "define.h"
#include "util/ctxpopup.h"
#include "util/util.h"

struct _selected_cb {
	void (*func)(void *data, int id);
	void *data;
};

struct ctxpopup {
	Evas_Object *popup;
	Evas_Object *focused_btn;

	struct _selected_cb cb;
};

static void _key_down_cb(int id, void *data, Evas *e, Evas_Object *obj,
			Evas_Event_Key_Down *ev)
{
	struct ctxpopup *m;

	if (!data || !ev)
		return;

	m = data;

	if (!strcmp(ev->keyname, KEY_BACK) ||
		!strcmp(ev->keyname, KEY_ESC))
		ctxpopup_destroy(m);
}

static void _mouse_move_cb(int id, void *data, Evas *e, Evas_Object *obj,
			Evas_Event_Mouse_Move *ev)
{
	if (!obj)
		return;

	if (!elm_object_focus_get(obj))
		elm_object_focus_set(obj, EINA_TRUE);
}

static void _clicked_cb(int id, void *data, Evas_Object *obj)
{
	struct ctxpopup *m;

	if (!data)
		return;

	m = data;

	if (m->cb.func)
		m->cb.func(m->cb.data, id);

	ctxpopup_destroy(m);
}

static void _dismissed_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct ctxpopup *m;

	if (!data)
		return;

	m = data;

	ctxpopup_destroy(m);
}

static input_handler _handler = {
	.key_down = _key_down_cb,
	.mouse_move = _mouse_move_cb,
	.clicked = _clicked_cb
};

void ctxpopup_set_callback(struct ctxpopup *m, void (*func)(void *, int),
			void *data)
{
	if (!m) {
		_ERR("failed to get ctxpopup info");
		return;
	}

	m->cb.func = func;
	m->cb.data = data;
}

void ctxpopup_show(struct ctxpopup *m, int x, int y)
{
	if (!m) {
		_ERR("failed to get ctxpopup info");
		return;
	}

	if (m->focused_btn)
		elm_object_focus_set(m->focused_btn, EINA_TRUE);

	evas_object_move(m->popup, x * elm_config_scale_get(),
				y * elm_config_scale_get());

	evas_object_show(m->popup);
}

struct ctxpopup *ctxpopup_create(Evas_Object *base, const char *ctxpopup_style,
			const char *btn_style, const char **text, int text_size,
			int cur_index)
{
	struct ctxpopup *m;
	Evas_Object *ctxpopup, *box, *btn;
	Evas_Object *firstbtn;
	Evas_Object *focused_btn;
	int i;

	if (!base || !text || text_size <= 0) {
		_ERR("invalid argument");
		return NULL;
	}

	m = calloc(1, sizeof(*m));
	if (!m) {
		_ERR("failed to allocate");
		return NULL;
	}

	ctxpopup = elm_ctxpopup_add(base);
	if (!ctxpopup) {
		_ERR("failed to create ctxpopup object");
		free(m);
		return NULL;
	}

	if (ctxpopup_style)
		elm_object_style_set(ctxpopup, ctxpopup_style);

	box = util_add_box(ctxpopup, EINA_FALSE);
	if (!box) {
		_ERR("failed to add box");
		evas_object_del(ctxpopup);
		free(m);
		return false;
	}

	elm_box_padding_set(box, 0, 0);

	focused_btn = NULL;
	for (i = 0; i < text_size; i++) {
		btn = util_add_button(box, btn_style, text[i]);
		if (!btn) {
			_ERR("failed to add button");
			evas_object_del(ctxpopup);
			free(m);
			return false;
		}

		elm_box_pack_end(box, btn);

		evas_object_show(btn);

		inputmgr_add_callback(btn, i, &_handler, m);

		if (i == 0)
			firstbtn = btn;

		if (i == cur_index)
			focused_btn = btn;
	}

	elm_object_focus_next_object_set(firstbtn, btn, ELM_FOCUS_UP);
	elm_object_focus_next_object_set(btn, firstbtn, ELM_FOCUS_DOWN);

	elm_object_content_set(ctxpopup, box);

	evas_object_smart_callback_add(ctxpopup, SIG_DISMISSED,
				_dismissed_cb, m);

	m->popup = ctxpopup;
	m->focused_btn = focused_btn;

	return m;
}

void ctxpopup_destroy(struct ctxpopup *m)
{
	if (!m) {
		_ERR("failed to get ctxpopup info");
		return;
	}

	evas_object_hide(m->popup);
	evas_object_del(m->popup);

	free(m);
}
