/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <app.h>
#include <Elementary.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <inputmgr.h>
#include <media_content.h>
#include <media_info.h>
#include <app_media.h>

#include "define.h"
#include "view.h"
#include "util/controller.h"
#include "util/util.h"

#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))

#define STYLE_ARROW_BTN "zoom_btn"
#define PART_ARROW_BTN "arrow_btn"

#define ZOOM_INTERVAL 0.3
#define ZOOM_RATIO_X2 0.5
#define ZOOM_RATIO_X4 0.25
#define MOVE_SIZE 10

#define TOAST_MESSAGE "Press Back to Escape from Zoom mode"
#define TOAST_TIMEOUT 3.0

#define NAVI_RECT_NUM 5

enum {
	DIR_UP = 0,
	DIR_DOWN,
	DIR_LEFT,
	DIR_RIGHT,
	DIR_NUM
};

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *photo;
	Evas_Object *thumb;
	Evas_Object *navi[NAVI_RECT_NUM];
	Evas_Object *notify;
	app_media_info *mi;
	struct controller *ctl;
	double image_ratio;
	double zoom_ratio;
	int navi_w;
	int navi_h;
	bool drag;
};

struct _btn_info {
	const char *name;
	int loc;
};

static struct _btn_info btn_arrow[DIR_NUM] = {
	{
		.name = SRC_BTN_UP,
		.loc = DIR_UP,
	},
	{
		.name = SRC_BTN_DOWN,
		.loc = DIR_DOWN,
	},
	{
		.name = SRC_BTN_LEFT,
		.loc = DIR_LEFT,
	},
	{
		.name = SRC_BTN_RIGHT,
		.loc = DIR_RIGHT,
	},
};

static void _hide_toast(struct _priv *priv)
{
	if (priv->notify)
		evas_object_hide(priv->notify);
}

static void _show_toast(struct _priv *priv)
{
	Evas_Object *notify;

	notify = util_add_notify(priv->base, STYLE_TOAST, STYLE_TOAST,
			TOAST_MESSAGE, TOAST_TIMEOUT);
	if (!notify) {
		_ERR("failed to add notify object");
		return;
	}

	priv->notify = notify;
}

static app_media_info *_get_media_info(Eina_List *list, int cur)
{
	app_media *am;
	app_media_info *mi;

	am = eina_list_nth(list, cur);
	if (!am) {
		_ERR("failed to get app_media");
		return NULL;
	}

	mi = app_media_get_info(am);
	if (!mi) {
		_ERR("failed to getting media info");
		return NULL;
	}

	return mi;
}

static void _get_image_ratio(struct _priv *priv)
{
	Evas_Object *obj;
	int w, w2;

	elm_photocam_image_size_get(priv->photo, &w, NULL);

	obj = elm_image_object_get(priv->thumb);
	evas_object_geometry_get(obj, NULL, NULL, &w2, NULL);

	priv->image_ratio = w2 / (double)w;
}

static void _get_zoom_ratio(struct _priv *priv)
{
	app_image_info *image;
	int win, img;
	int w, h;
	double ratio;

	elm_win_screen_size_get(priv->win, NULL, NULL, &w, &h);

	image = priv->mi->image;

	if (image->width > image->height) {
		img = image->width;
		win = w;
		ratio = ZOOM_RATIO_X2;
	} else {
		img = image->height;
		win = h;
		ratio = ZOOM_RATIO_X4;
	}

	priv->zoom_ratio = ratio * img / (double)win;
}

static void _update_navi(struct _priv *priv)
{
	Evas_Object *obj;
	int px, py, pw, ph;	/* photo's geometry */
	int nx, ny, nw, nh;	/* thumb's geometry */
	int mx, my, mw, mh;	/* navi's geometry */
	int x[DIR_NUM], y[DIR_NUM], w[DIR_NUM], h[DIR_NUM];
	int i;

	elm_photocam_image_region_get(priv->photo, &px, &py, &pw, &ph);

	obj = elm_image_object_get(priv->thumb);
	evas_object_geometry_get(obj, &nx, &ny, &nw, &nh);

	mx = nx + px * priv->image_ratio;
	my = ny + py * priv->image_ratio;
	mw = pw * priv->image_ratio;
	mh = ph * priv->image_ratio;

	priv->navi_w = mw;
	priv->navi_h = mh;

	/* up */
	x[DIR_UP] = nx;
	y[DIR_UP] = ny;
	w[DIR_UP] = nw;
	h[DIR_UP] = my - ny;

	/* down */
	x[DIR_DOWN] = nx;
	y[DIR_DOWN] = my + mh;
	w[DIR_DOWN] = nw;
	h[DIR_DOWN] = nh - mh - h[DIR_UP];

	/* left */
	x[DIR_LEFT] = nx;
	y[DIR_LEFT] = my;
	w[DIR_LEFT] = mx - nx;
	h[DIR_LEFT] = mh;

	/* right */
	x[DIR_RIGHT] = mx + mw;
	y[DIR_RIGHT] = my;
	w[DIR_RIGHT] = nw - mw - w[DIR_LEFT];
	h[DIR_RIGHT] = mh;

	for (i = 0; i < DIR_NUM; i++) {
		evas_object_resize(priv->navi[i], w[i], h[i]);
		evas_object_move(priv->navi[i], x[i], y[i]);
	}

	evas_object_resize(priv->navi[i], mw, mh);
	evas_object_move(priv->navi[i], mx, my);
}

static void _move_navi(struct _priv *priv, Evas_Coord_Point coord)
{
	Evas_Object *obj;
	int px, py, pw, ph;	/* photo's geometry */
	int nx, ny, nw, nh;	/* thumb's geometry */
	int mx, my, mw, mh;	/* navi's geometry */
	int w, h;

	elm_photocam_image_size_get(priv->photo, &w, &h);

	elm_photocam_image_region_get(priv->photo, &px, &py, &pw, &ph);

	obj = elm_image_object_get(priv->thumb);
	evas_object_geometry_get(obj, &nx, &ny, &nw, &nh);

	mw = priv->navi_w;
	mh = priv->navi_h;

	mx = coord.x - (mw / 2);
	my = coord.y - (mh / 2);

	if (mx < nx)
		mx = 0;
	else if (mx > nx + nw - mw)
		mx = w - pw;
	else
		mx = (mx - nx) / priv->image_ratio;

	if (my < ny)
		my = 0;
	else if (my > ny + nh - mh)
		my = h - ph;
	else
		my = (my - ny) / priv->image_ratio;

	elm_scroller_region_show(priv->photo, mx, my, pw, ph);
}

static Eina_Bool _zoom_set(void *data)
{
	struct _priv *priv;

	if (!data)
		return ECORE_CALLBACK_CANCEL;

	priv = data;

	_get_zoom_ratio(priv);

	elm_photocam_zoom_mode_set(priv->photo, ELM_PHOTOCAM_ZOOM_MODE_MANUAL);
	elm_photocam_zoom_set(priv->photo, priv->zoom_ratio);

	_get_image_ratio(priv);
	_update_navi(priv);

	return ECORE_CALLBACK_CANCEL;
}

static void _scrolled(void *data, Evas_Object *obj, void *ev)
{
	struct _priv *priv;

	if (!data)
		return;

	priv = data;

	_update_navi(priv);
}

static bool _move_image(struct _priv *priv, int dir)
{
	int x, y, w, h;
	int px, py;

	elm_scroller_region_get(priv->photo, &x, &y, &w, &h);

	switch (dir) {
	case DIR_UP:
		y -= MOVE_SIZE;
		break;
	case DIR_DOWN:
		y += MOVE_SIZE;
		break;
	case DIR_LEFT:
		x -= MOVE_SIZE;
		break;
	case DIR_RIGHT:
		x += MOVE_SIZE;
		break;
	default:
		return false;
	}

	elm_scroller_region_show(priv->photo, x, y, w, h);

	elm_scroller_region_get(priv->photo, &px, &py, NULL, NULL);
	if (x != px || y != py)
		return false;

	return true;
}

static void _key_up(int id, void *data, Evas *e, Evas_Object *obj,
			Evas_Event_Key_Up *ev)
{
	struct _priv *priv;
	struct controller *ctl;

	if (!data || !ev)
		return;

	priv = data;
	ctl = priv->ctl;

	if (!strcmp(ev->keyname, KEY_BACK) ||
			!strcmp(ev->keyname, KEY_ESC))
		viewmgr_pop_view();
	else if (!strcmp(ev->keyname, KEY_UP))
		ctl->ops->signal(ctl->handle, DIR_UP, SIG_SET_UNFOCUS);
	else if (!strcmp(ev->keyname, KEY_DOWN))
		ctl->ops->signal(ctl->handle, DIR_DOWN, SIG_SET_UNFOCUS);
	else if (!strcmp(ev->keyname, KEY_LEFT))
		ctl->ops->signal(ctl->handle, DIR_LEFT, SIG_SET_UNFOCUS);
	else if (!strcmp(ev->keyname, KEY_RIGHT))
		ctl->ops->signal(ctl->handle, DIR_RIGHT, SIG_SET_UNFOCUS);
}

static void _key_down(int id, void *data, Evas *e, Evas_Object *obj,
			Evas_Event_Key_Down *ev)
{
	struct _priv *priv;
	struct controller *ctl;
	const char *sig;
	int dir;

	if (!data || !ev)
		return;

	priv = data;
	ctl = priv->ctl;

	if (!strcmp(ev->keyname, KEY_UP))
		dir = DIR_UP;
	else if (!strcmp(ev->keyname, KEY_DOWN))
		dir = DIR_DOWN;
	else if (!strcmp(ev->keyname, KEY_LEFT))
		dir = DIR_LEFT;
	else if (!strcmp(ev->keyname, KEY_RIGHT))
		dir = DIR_RIGHT;
	else
		return;

	if (_move_image(priv, dir))
		sig = SIG_SET_FOCUS;
	else
		sig = SIG_SET_UNFOCUS;

	ctl->ops->signal(ctl->handle, dir, sig);
}

static void _mouse_move(int id, void *data, Evas *e, Evas_Object *obj,
			Evas_Event_Mouse_Move *ev)
{
	struct _priv *priv;

	if (!data || !ev)
		return;

	priv = data;

	if (!priv->drag)
		return;

	_move_navi(priv, ev->cur.canvas);
}

static void _mouse_up(int id, void *data, Evas *e, Evas_Object *obj,
			Evas_Event_Mouse_Up *ev)
{
	struct _priv *priv;

	if (!data || !ev)
		return;

	priv = data;

	priv->drag = false;
}

static void _mouse_down(int id, void *data, Evas *e, Evas_Object *obj,
			Evas_Event_Mouse_Down *ev)
{
	struct _priv *priv;

	if (!data || !ev)
		return;

	priv = data;

	priv->drag = true;

	_move_navi(priv, ev->canvas);
}

static input_handler _navi_handler = {
	.mouse_move = _mouse_move,
	.mouse_down = _mouse_down,
	.mouse_up = _mouse_up,
};

static input_handler _btn_handler = {
	.key_up = _key_up,
	.key_down = _key_down,
};

static bool _draw_image(struct _priv *priv)
{
	Evas_Object *obj;

	obj = elm_photocam_add(priv->base);
	if (!obj) {
		_ERR("failed to adding photocam");
		return false;
	}

	priv->photo = obj;

	elm_object_focus_allow_set(obj, EINA_FALSE);
	elm_scroller_policy_set(obj,
			ELM_SCROLLER_POLICY_OFF,
			ELM_SCROLLER_POLICY_OFF);
	elm_photocam_file_set(obj, priv->mi->file_path);
	elm_photocam_paused_set(obj, EINA_TRUE);

	elm_object_part_content_set(priv->base, PART_ZOOM_CONTENT, obj);

	evas_object_smart_callback_add(obj, SIG_SCROLL, _scrolled, priv);

	return true;
}

static bool _draw_thumbnail(struct _priv *priv)
{
	Evas_Object *obj;

	obj = elm_image_add(priv->base);
	if (!obj) {
		_ERR("failed to adding photocam");
		return false;
	}

	priv->thumb = obj;

	evas_object_size_hint_align_set(obj, 1.0, 1.0);
	elm_image_file_set(obj, priv->mi->file_path, NULL);

	elm_object_part_content_set(priv->base, PART_ZOOM_NAVI, obj);

	return true;
}

static bool _draw_navi(struct _priv *priv)
{
	Evas_Object *obj;
	int i;

	for (i = 0; i < NAVI_RECT_NUM; i++) {
		obj = evas_object_rectangle_add(
				evas_object_evas_get(priv->base));
		if (!obj) {
			_ERR("failed to add rect");
			return false;
		}

		if (i == NAVI_RECT_NUM - 1)
			evas_object_color_set(obj, 0, 0, 0, 0);
		else
			evas_object_color_set(obj, 0, 0, 0, 204);

		evas_object_show(obj);

		inputmgr_add_callback(obj, 0, &_navi_handler, priv);

		priv->navi[i] = obj;
	}

	return true;
}

static void _draw_contents(struct _priv *priv)
{
	if (!_draw_image(priv))
		return;

	if (!_draw_thumbnail(priv))
		return;

	if (!_draw_navi(priv))
		return;

	/* zoom should set after showing photocam object */
	ecore_timer_add(ZOOM_INTERVAL, _zoom_set, priv);
}

static bool _ui_init(struct _priv *priv)
{
	Evas_Object *obj;
	struct controller *ctl;
	int i;

	ctl = controller_create(priv->base);
	if (!ctl) {
		_ERR("failed to create controller");
		return false;
	}

	for (i = 0; i < DIR_NUM; i++) {
		ctl->ops->add_control(ctl->handle,
				btn_arrow[i].name, btn_arrow[i].loc,
				STYLE_ARROW_BTN, PART_ARROW_BTN);

		obj = ctl->ops->get_object(ctl->handle, i);
		if (obj)
			inputmgr_add_callback(obj, 0, &_btn_handler, priv);
	}

	priv->ctl = ctl;

	return true;
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	Evas_Object *base;
	bool r;

	if (!win) {
		_ERR("failed to get win object");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to allocate priv");
		return NULL;
	}

	base = elm_layout_add(win);
	if (!base) {
		_ERR("failed to create base object");
		goto err;
	}

	elm_layout_file_set(base, EDJEFILE, GRP_ZOOM_VIEW);

	evas_object_size_hint_weight_set(base,
			EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, base);

	priv->win = win;
	priv->base = base;

	r = _ui_init(priv);
	if (!r) {
		_ERR("failed to init UI");
		evas_object_del(base);
		goto err;
	}

	viewmgr_set_view_data(VIEW_ZOOM, priv);

	return base;

err:
	free(priv);
	return NULL;
}

static void _show(void *view_data)
{
	struct _priv *priv;
	struct controller *ctl;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	evas_object_show(priv->base);

	_draw_contents(priv);

	_show_toast(priv);

	ctl = priv->ctl;
	ctl->ops->show(ctl->handle);
	ctl->ops->focus(ctl->handle, DIR_UP, true);
}

static void _hide(void *view_data)
{
	struct _priv *priv;
	struct controller *ctl;
	int i;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	evas_object_hide(priv->base);

	if (priv->photo)
		evas_object_del(priv->photo);

	if (priv->thumb)
		evas_object_del(priv->thumb);

	for (i = 0; i < NAVI_RECT_NUM; i++) {
		if (priv->navi[i]) {
			inputmgr_remove_callback(priv->navi[i], &_navi_handler);
			evas_object_del(priv->navi[i]);
		}
	}

	_hide_toast(priv);

	ctl = priv->ctl;
	ctl->ops->hide(ctl->handle);
}

static void _update(void *view_data, int update_type, void *data)
{
	struct _priv *priv;
	struct view_update_data *vdata;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;
	vdata = data;

	switch (update_type) {
	case UPDATE_CONTENT:
		if (!vdata)
			break;

		priv->mi = _get_media_info(vdata->list, vdata->index);
		break;
	default:
		break;
	}
}

static void _pause(void *view_data)
{
	viewmgr_pop_view();
}

static void _destroy(void *view_data)
{
	struct _priv *priv;
	struct controller *ctl;
	Evas_Object *obj;
	int i;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	ctl = priv->ctl;

	for (i = 0; i < ARRAY_SIZE(btn_arrow); i++) {
		obj = ctl->ops->get_object(ctl->handle, i);
		if (obj)
			inputmgr_remove_callback(obj, &_btn_handler);
	}

	evas_object_del(priv->base);

	free(priv);
}

static view_class _vclass = {
	.view_id = VIEW_ZOOM,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.update = _update,
	.pause = _pause,
	.destroy = _destroy,
};

view_class *view_zoom_get_vclass(void)
{
	return &_vclass;
}
